# python 2 backwards compatibility
from __future__ import print_function
from builtins import object, super
from future import standard_library
from six import string_types

# package imports
from .base import ModelBase
from .intelligence_source import IntelligenceSource


class IndicatorSummary(ModelBase):
    """
    Models an |IndicatorSummary_resource|.  This represents a normalized summary of common properties extracted from the
    body of a report, from an intelligence source, that gives details about a specific indicator.

    The score field will only be populated if the source contained information that can be interpreted as a type of score.
    The attributes field is a list of |IndicatorAttribute| objects for fields that are specific to this source.

    :ivar str value: The indicator's value.
    :ivar IndicatorType indicator_type: The indicator's type.
    :ivar str report_id: The ID of the report for this summary.
    :ivar str enclave_id: The ID of the report's enclave.
    :ivar IntelligenceSource source: An object containing information about the source that the report came from.
    :ivar IndicatorScore score: The score of the report, according to the source.
    :ivar int created: The created or first seen timestamp of the indicator, according to the source.
    :ivar int updated: The updated or last seen timestamp of the indicator, according to the source.
    :ivar str description: The description of the indicator, according to the source.
    :ivar list(Attribute) attributes: A list of attributes about the indicator, according to the source.
    :ivar str severity_level: a normalized representation of the score from this source (if one exists).  This is
        an integer between 0 and 3, with 0 being the lowest score and 3 being the highest.
    """

    def __init__(self,
                 value=None,
                 indicator_type=None,
                 report_id=None,
                 enclave_id=None,
                 source=None,
                 score=None,
                 created=None,
                 updated=None,
                 description=None,
                 attributes=None,
                 severity_level=None):

        self.value = value
        self.indicator_type = indicator_type
        self.report_id = report_id
        self.enclave_id = enclave_id
        self.source = source
        self.score = score
        self.created = created
        self.updated = updated
        self.description = description
        self.attributes = attributes
        self.severity_level = severity_level

    @classmethod
    def from_dict(cls, indicator_summary):
        """
        Create an |IndicatorSummary| object from a dictionary.

        :param indicator_summary: The dictionary.
        :return: The |IndicatorSummary| object.
        """

        attributes = [IndicatorAttribute.from_dict(attribute) for attribute in indicator_summary.get('attributes', [])]

        source = indicator_summary.get('source')
        if source:
            source = IntelligenceSource.from_dict(indicator_summary.get('source'))
            
        score = indicator_summary.get('score')
        if score:
            score = IndicatorScore.from_dict(indicator_summary.get('score'))

        return IndicatorSummary(value=indicator_summary.get('value'),
                                indicator_type=indicator_summary.get('type'),
                                report_id=indicator_summary.get('reportId'),
                                enclave_id=indicator_summary.get('enclaveId'),
                                source=source,
                                score=score,
                                created=indicator_summary.get('created'),
                                updated=indicator_summary.get('updated'),
                                description=indicator_summary.get('description'),
                                attributes=attributes,
                                severity_level=indicator_summary.get('severityLevel'))

    def to_dict(self, remove_nones=False):
        """
        Creates a dictionary representation of the indicator summary.

        :param remove_nones: Whether ``None`` values should be filtered out of the dictionary.  Defaults to ``False``.
        :return: A dictionary representation of the indicator summary.
        """

        if remove_nones:
            return super().to_dict(remove_nones=True)

        source = None
        if self.source is not None:
            source = self.source.to_dict()

        score = None
        if self.score is not None:
            score = self.score.to_dict()

        attributes = None
        if self.attributes is not None:
            attributes = [attribute.to_dict(remove_nones=remove_nones) for attribute in self.attributes]

        return {
            'value': self.value,
            'type': self.indicator_type,
            'reportId': self.report_id,
            'enclaveId': self.enclave_id,
            'source': source,
            'score': score,
            'created': self.created,
            'updated': self.updated,
            'description': self.description,
            'attributes': attributes,
            'severityLevel': self.severity_level
        }


class IndicatorScore(ModelBase):
    """
    Models a |IndicatorScore_resource|.

    :ivar str name: The name of the score type, e.g. "Risk Score" or "Malicious Confidence"
    :ivar str value: The value of the score, as directly extracted from the source.
    """

    def __init__(self,
                 name=None,
                 value=None):

        self.name = name
        self.value = value

    @classmethod
    def from_dict(cls, indicator_score):
        """
        Create an |IndicatorScore| object from a dictionary.

        :param indicator_score: The dictionary.
        :return: The |IndicatorScore| object.
        """

        return IndicatorScore(name=indicator_score.get('name'),
                              value=indicator_score.get('value'))

    def to_dict(self, remove_nones=False):
        """
        Creates a dictionary representation of the indicator score.

        :param remove_nones: Whether ``None`` values should be filtered out of the dictionary.  Defaults to ``False``.
        :return: A dictionary representation of the indicator score.
        """

        return {
            'name': self.name,
            'value': self.value
        }


class IndicatorAttribute(ModelBase):
    """
    Models a |IndicatorAttribute_resource|.  This is an attribute of an indicator, according to an intelligence source.

    :ivar str name: The name of the attribute, e.g. "Actors" or "Malware Families"
    :ivar any value: The value of the attribute, e.g. "North Korea" or "Emotet"
    :ivar str logical_type: Describes how to interpret the ``value`` field, e.g. could be "timestamp" if ``value`` is an integer
    :ivar str description: A description of how to interpret this attribute.  This corresponds to the attribute name,
        i.e. this will be the same for all attributes in a source with the same name.
    """

    def __init__(self,
                 name=None,
                 value=None,
                 logical_type=None,
                 description=None):

        self.name = name
        self.value = value
        self.logical_type = logical_type
        self.description = description

    @classmethod
    def from_dict(cls, indicator_attribute):
        """
        Create an |IndicatorAttribute| object from a dictionary.

        :param indicator_attribute: The dictionary.
        :return: The |IndicatorAttribute| object.
        """

        return IndicatorAttribute(name=indicator_attribute.get('name'),
                                  value=indicator_attribute.get('value'),
                                  logical_type=indicator_attribute.get('logicalType'),
                                  description=indicator_attribute.get('description'))

    def to_dict(self, remove_nones=False):
        """
        Creates a dictionary representation of the indicator attribute.

        :param remove_nones: Whether ``None`` values should be filtered out of the dictionary.  Defaults to ``False``.
        :return: A dictionary representation of the indicator attribute.
        """

        return {
            'name': self.name,
            'value': self.value,
            'logicalType': self.logical_type,
            'description': self.description
        }



# python 2 backwards compatibility
from __future__ import print_function
from builtins import object, str, super
from future import standard_library
from six import string_types

import logging
from logging.config import dictConfig
from .utils import parse_boolean
import os
import sys


class InfoFilter(logging.Filter):
    """
    Filters log messages based on whether their level is less than or equal to INFO.
    """

    def __init__(self, below):
        super().__init__()
        self.below = below

    def filter(self, rec):
        less_than = rec.levelno <= logging.INFO
        return less_than if self.below else not less_than


DEFAULT_LOGGING_CONFIG = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'infoFilter': {
            '()': InfoFilter,
            'below': True
        },
        'errorFilter': {
            '()': InfoFilter,
            'below': False
        }
    },
    'formatters': {
        'detailed': {
            'class': 'logging.Formatter',
            'format': '%(asctime)s - %(levelname)-7s - %(name)s:%(lineno)d - %(message)s'
        }
    },
    'handlers': {
        'console': {
            'class': 'logging.StreamHandler',
            'level': 'INFO',
            'formatter': 'detailed',
            'filters': ['infoFilter'],
            'stream': 'ext://sys.stdout',
        },
        'error_console': {
            'class': 'logging.StreamHandler',
            'level': 'INFO',
            'formatter': 'detailed',
            'filters': ['errorFilter'],
            'stream': 'ext://sys.stderr',
        },
    },
    'root': {
        'level': 'INFO',
        'handlers': [
            'console',
            'error_console'
        ]
    }
}


def configure_logging():
    """
    Initialize logging configuration to defaults.  If the environment variable DISABLE_TRUSTAR_LOGGING is set to true,
    this will be ignored.
    """

    if not parse_boolean(os.environ.get('DISABLE_TRUSTAR_LOGGING')):

        # configure
        dictConfig(DEFAULT_LOGGING_CONFIG)

        # construct error logger
        error_logger = logging.getLogger("error")

        # log all uncaught exceptions
        def log_exception(exc_type, exc_value, exc_traceback):
            error_logger.error("Uncaught exception", exc_info=(exc_type, exc_value, exc_traceback))

        # register logging function as exception hook
        sys.excepthook = log_exception



# python 2 backwards compatibility
from __future__ import print_function
from builtins import object, str
from future import standard_library
from six import string_types
import logging
import json

# package imports
from .models import Tag

# python 2 backwards compatibility
standard_library.install_aliases()

logger = logging.getLogger(__name__)


class TagClient(object):
    
    def get_enclave_tags(self, report_id, id_type=None):
        """
        Retrieves all enclave tags present in a specific report.

        :param report_id: the ID of the report
        :param id_type: indicates whether the ID is an internal or external ID
        :return: A list of |Tag| objects.
        """

        params = {'idType': id_type}
        resp = self._client.get("reports/%s/tags" % report_id, params=params)
        return [Tag.from_dict(indicator) for indicator in resp.json()]

    def alter_report_tags(self, report_id, added_tags, removed_tags, id_type=None):
        """
        Bulk add/remove tags from a report.

        :param report_id: the ID of the report
        :param added_tags: a list of strings, the names of tags to add
        :param removed_tags: a list of strings, the names of tags to remove
        :return: the ID of the report
        """

        params = {'idType': id_type}
        body = {
            'addedTags': [{'name': tag_name} for tag_name in added_tags],
            'removedTags': [{'name': tag_name} for tag_name in removed_tags]
        }

        resp = self._client.post("reports/{}/alter-tags".format(report_id), params=params, data=json.dumps(body))
        return resp.json().get('id')

    def add_enclave_tag(self, report_id, name, enclave_id=None, id_type=None):
        """
        Adds a tag to a specific report, for a specific enclave.

        NOTE: This function is deprecated.  Use alter_report_tags instead.

        :param report_id: The ID of the report
        :param name: The name of the tag to be added
        :param enclave_id: ID of the enclave where the tag will be added.
            NOTE: This field is deprecated. Report tags are no longer enclave-specific.
            This field will be ignored if it is filled out.
        :param id_type: indicates whether the ID is an internal or external ID
        :return: The ID of the tag that was created.
        """

        # delegate to alter-tags endpoint
        self.alter_report_tags(report_id=report_id,
                               added_tags=[name],
                               removed_tags=[],
                               id_type=id_type)

        # return the tag name to maintain backwards compatibility
        # the old endpoint returned the tag ID, but tag IDs no longer exist
        # the tag name is now used in place of its ID throughout all API endpoints
        return name

    def delete_enclave_tag(self, report_id, tag_id, id_type=None):
        """
        Deletes a tag from a specific report, in a specific enclave.

        NOTE: This function is deprecated.  Use alter_report_tags instead.

        :param string report_id: The ID of the report
        :param string tag_id: name of the tag to delete.
            NOTE: Report tags no longer have IDs, instead the name of the tag serves as its ID.
            Pass the name of the tag to delete in this field.
        :param string id_type: indicates whether the ID internal or an external ID provided by the user
        :return: The ID of the report.
        """

        # delegate to alter-tags endpoint
        return self.alter_report_tags(report_id=report_id,
                                      added_tags=[],
                                      removed_tags=[tag_id],
                                      id_type=id_type)

    def get_all_enclave_tags(self, enclave_ids=None):
        """
        Retrieves all tags present in the given enclaves. If the enclave list is empty, the tags returned include all
        tags for all enclaves the user has access to.

        :param (string) list enclave_ids: list of enclave IDs
        :return: The list of |Tag| objects.
        """

        params = {'enclaveIds': enclave_ids}
        resp = self._client.get("reports/tags", params=params)
        return [Tag.from_dict(indicator) for indicator in resp.json()]

    def get_all_indicator_tags(self, enclave_ids=None):
        """
        Get all indicator tags for a set of enclaves.

        :param (string) list enclave_ids: list of enclave IDs
        :return: The list of |Tag| objects.
        """

        if enclave_ids is None:
            enclave_ids = self.enclave_ids

        params = {'enclaveIds': enclave_ids}
        resp = self._client.get("indicators/tags", params=params)
        return [Tag.from_dict(indicator) for indicator in resp.json()]

    def add_indicator_tag(self, indicator_value, name, enclave_id):
        """
        Adds a tag to a specific indicator, for a specific enclave.

        :param indicator_value: The value of the indicator
        :param name: The name of the tag to be added
        :param enclave_id: ID of the enclave where the tag will be added
        :return: A |Tag| object representing the tag that was created.
        """

        data = {
            'value': indicator_value,
            'tag': {
                'name': name,
                'enclaveId': enclave_id
            }
        }

        resp = self._client.post("indicators/tags", data=json.dumps(data))
        return Tag.from_dict(resp.json())

    def delete_indicator_tag(self, indicator_value, tag_id):
        """
        Deletes a tag from a specific indicator, in a specific enclave.

        :param indicator_value: The value of the indicator to delete the tag from
        :param tag_id: ID of the tag to delete
        """

        params = {
            'value': indicator_value
        }

        self._client.delete("indicators/tags/%s" % tag_id, params=params)



from __future__ import absolute_import

from .logger import configure_logging
configure_logging()

from .trustar import TruStar
from .models import *
from .utils import *

from .version import __version__, __api_version__



# python 2 backwards compatibility
from __future__ import print_function
from builtins import super

# package imports
from .base import ModelBase


class PhishingSubmission(ModelBase):
    """
    Models a PhishingSubmission

    ``context`` is a dictionary containing these fields:
    indicatorType, indicatorValue, sourceKey, normalizedIndicatorScore

    :ivar submission_id: The id of the email submission
    :ivar title: The title of the email submission (email subject)
    :ivar priority_event_score: The score of the email submission
    :ivar status: The current triage status of a submission ("UNRESOLVED", "CONFIRMED", or "IGNORED")
    :ivar context: A list containing dicts which represent IOCs, sources, and scores
                    that contributed to to the triage score.
    """

    def __init__(self,
                 submission_id=None,
                 title=None,
                 priority_event_score=None,
                 status=None,
                 context=None):
        """
        Constructs a PhishingSubmission object.

        """
        self.submission_id = submission_id
        self.title = title
        self.priority_event_score = priority_event_score
        self.status = status
        self.context = context

    @classmethod
    def from_dict(cls, phishing_submission):
        """
        Creates a phishing submission object from a dictionary.

        :param phishing_submission: The phishing submission dictionary.
        :return: The PhishingSubmission object.
        """

        return PhishingSubmission(submission_id=phishing_submission.get('submissionId'),
                                  title=phishing_submission.get('title'),
                                  priority_event_score=phishing_submission.get('priorityEventScore'),
                                  status=phishing_submission.get('status'),
                                  context=phishing_submission.get('context'))

    def to_dict(self, remove_nones=False):
        """
        Creates a dictionary representation of a phishing submission.

        :param remove_nones: Whether ``None`` values should be filtered out of the dictionary.  Defaults to ``False``.
        :return: A PhishingSubmission object.
        """

        if remove_nones:
            return super().to_dict(remove_nones=True)

        return {
            'submissionId': self.submission_id,
            'title': self.title,
            'priorityEventScore': self.priority_event_score,
            'status': self.status,
            'context': self.context,
        }


class PhishingIndicator(ModelBase):
    """
    Models a PhishingIndicator.

    :ivar indicator_type: The type of the extracted entity (e.g. URL, IP, ...)
    :ivar value: The value of an extracted entity (e.g. www.badsite.com, etc.)
    :ivar source_key: A string that is associated with the closed source providing context
                       (e.g. 'virustotal', 'crowdstrike_indicator')
    :ivar normalized_indicator_score: The normalized score associated with a context entity
    :ivar original_indicator_score: A score given to the indicator by its original source
    """

    def __init__(self,
                 indicator_type=None,
                 value=None,
                 source_key=None,
                 normalized_indicator_score=None,
                 original_indicator_score=None):
        """
        Constructs a PhishingIndicator object.
        """
        self.indicator_type = indicator_type
        self.value = value
        self.source_key = source_key
        self.normalized_indicator_score = normalized_indicator_score
        self.original_indicator_score = original_indicator_score

    @classmethod
    def from_dict(cls, phishing_indicator):
        """
        Creates a phishing indicator object from a dictionary.

        :param phishing_indicator: The phishing indicator dictionary.
        """

        return PhishingIndicator(indicator_type=phishing_indicator.get('indicatorType'),
                                 value=phishing_indicator.get('value'),
                                 source_key=phishing_indicator.get('sourceKey'),
                                 normalized_indicator_score=phishing_indicator.get('normalizedIndicatorScore'),
                                 original_indicator_score=phishing_indicator.get('originalIndicatorScore'))

    def to_dict(self, remove_nones=False):
        """
        Creates a dictionary representation of a phishing indicator.

        :param remove_nones: Whether ``None`` values should be filtered out of the dictionary.  Defaults to ``False``.
        :return: A PhishingIndicator object.
        """

        if remove_nones:
            return super().to_dict(remove_nones=True)

        return {
            'indicatorType': self.indicator_type,
            'value': self.value,
            'sourceKey': self.source_key,
            'normalizedIndicatorScore': self.normalized_indicator_score,
            'originalIndicatorScore': self.original_indicator_score
        }



import unittest
from trustar import *

import json
import time
import random
import requests
from unittest.mock import patch

from trustar.api_client import ApiClient

DAY = 24 * 60 * 60 * 1000

current_time = int(time.time()) * 1000
yesterday_time = current_time - DAY
old_time = current_time - DAY * 365 * 3
MILLISECONDS_IN_A_DAY = 24 * 60 * 60 * 1000

# Default Phishing Triage parameters to include all options
normalized_triage_scores = [3, 2, 1]
normalized_source_scores = [3, 2, 1]
statuses = ["UNRESOLVED", "CONFIRMED", "IGNORED"]


def generate_ip(start_range=100):
    """
    Generates a random IP where each.
    :param start_range: The lowest possible value for each octet.
    :return: The random IP address.
    """
    return ".".join(map(str, (random.randint(start_range, 255) for _ in range(4))))


class TruStarTests(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.ts = TruStar(config_role='staging')

    def test_ping(self):
        pong = self.ts.ping()
        self.assertTrue(len(pong) > 0)

    def test_version(self):
        version = self.ts.get_version()
        BETA_TAG = "-beta"
        self.assertEqual(version.strip(BETA_TAG), trustar.__api_version__.strip(BETA_TAG))

    def test_get_reports(self):
        """
        Test that we can get all reports fitting some filters.
        """
        for from_time in [yesterday_time, old_time]:
            reports = self.ts.get_reports_page(from_time=from_time,
                                               to_time=current_time,
                                               is_enclave=False)
            print(reports)

    def test_submit_report(self):
        """
        Test the entire workflow of submitting, updating, getting, tagging, and deleting a report.
        """
        # create and submit report
        report = Report(title="Report 1",
                        body="Blah blah blah",
                        time_began=yesterday_time,
                        enclave_ids=self.ts.enclave_ids)
        report = self.ts.submit_report(report=report)

        # update report
        report.body = "Bleh bleh bleh"
        self.ts.update_report(report=report)

        # get report
        result = self.ts.get_report_details(report_id=report.id)
        self.assertEqual(result.body, report.body)

        # add tag
        result = self.ts.add_enclave_tag(report_id=report.id,
                                         enclave_id=self.ts.enclave_ids[0],
                                         name="some_tag")

        # get tags
        result = self.ts.get_enclave_tags(report_id=report.id)
        tag_id = None
        for tag in result:
            if tag.name == "some_tag":
                tag_id = tag.id

        self.assertIsNotNone(tag_id)

        # delete tag
        result = self.ts.delete_enclave_tag(report_id=report.id,
                                            tag_id=tag_id)

        # delete report
        response = self.ts.delete_report(report_id=report.id)

    def test_community_trends(self):
        """
        Check that we can get community trending indicators, and that the total
        number of each indicator type (except MALWARE and CVE) is consistent with
        the total number returned when we get ALL indicator types (except MALWARE and CVE).
        """
        totals = []

        indicator_types = {
            IndicatorType.CVE,
            IndicatorType.MALWARE,
            None
        }

        # get results for all types and sum total elements for all except CVE and MALWARE
        for indicator_type in indicator_types:
            result = self.ts.get_community_trends(indicator_type=indicator_type)

            if indicator_type is not None:
                # ensure only indicators of correct type received
                for indicator in result:
                    self.assertEqual(indicator.type, indicator_type)

            correlation_counts = [indicator.correlation_count for indicator in result]
            for i in range(len(correlation_counts) - 1):
                self.assertTrue(correlation_counts[i] >= correlation_counts[i + 1])

    def test_get_related_indicators_and_correlated_reports(self):
        """
        Submits a group of reports that contain common indicators, then checks that
        getting related indicators and correlated reports returns the expected results.
        """

        #########
        # SETUP #
        #########

        indicators = [
            "www.wefrrtdgwefwef1234.org",
            "wefoijseroijr@yahoo.org"
        ]

        related = ["www.abcxyz1235.com"] + [generate_ip() for _ in range(27)]

        indicator_groups = [
            related[:5],
            related[5:9],
            related[9:19],
            related[19:25],
            related[25:]
        ]

        reports = []

        count = 0
        for group in indicator_groups:
            count += 1
            report = Report(
                title="Test_Get_Related_Indicators_Report_%s" % count,
                body=" some words ".join([indicators[count % 2]] + group),
                enclave_ids=self.ts.enclave_ids
            )
            report = self.ts.submit_report(report=report)
            reports.append(report)

        count = 0
        for report, indicator_group in zip(reports, indicator_groups):
            count += 1
            report_indicators = self.ts.get_indicators_for_report(report_id=report.id)
            self.assertSetEqual(set(str(x.value) for x in report_indicators),
                                set([indicators[count % 2]] + indicator_group))

        ###############
        # GET RELATED #
        ###############

        server_related = list(self.ts.get_related_indicators(indicators=indicators))
        related_reports = self.ts.get_correlated_report_ids(indicators=indicators)

        ###########
        # CLEANUP #
        ###########

        for report in reports:
            self.ts.delete_report(report_id=report.id)

        ##########
        # ASSERT #
        ##########

        server_indicator_values = set([str(ind.value.lower()) for ind in server_related])
        for indicator in related:
            self.assertTrue(indicator.lower() in server_indicator_values)

    def test_get_related_indicators(self):
        result = self.ts.get_related_indicators_page(indicators=["evil", "1.2.3.4", "wannacry"])

    def test_get_correlated_reports(self):
        result = self.ts.get_correlated_report_ids(["evil", "wannacry"])

    def test_get_reports_by_tag(self):
        """
        Test workflow of submitting and tagging a report, then getting all reports by that tag name.
        """

        from_time = get_current_time_millis()

        enclave_id = self.ts.enclave_ids[0]

        # create and submit report
        report = Report(title="Report 1",
                        body="Blah blah blah",
                        time_began=yesterday_time,
                        enclave_ids=[enclave_id])
        report = self.ts.submit_report(report=report)

        # tag report
        tag = "some gibberish"
        self.ts.add_enclave_tag(report_id=report.id, name=tag, enclave_id=enclave_id)

        # sleep to allow time to process
        time.sleep(5)

        # get all reports with the tag just created
        to_time = get_current_time_millis()
        report_ids = [r.id for r in self.ts.get_reports(tag=tag, from_time=from_time, to_time=to_time)]

        try:
            # assert that only the report submitted earlier was found
            self.assertEqual(len(report_ids), 1)
            self.assertEqual(report_ids[0], report.id)
        finally:
            # cleanup reports
            for id in report_ids:
                self.ts.delete_report(report_id=id)

    @unittest.skip
    def test_search_indicators(self):
        indicators = self.ts.search_indicators("abc")
        self.assertGreater(len(list(indicators)), 0)

    @unittest.skip
    def test_search_reports(self):
        reports = self.ts.search_reports("abc")
        self.assertGreater(len(list(reports)), 0)

    def test_get_enclaves(self):
        enclaves = self.ts.get_user_enclaves()
        self.assertGreater(len(enclaves), 0)
        print(enclaves[0])

    def test_whitelist(self):
        indicators = self.ts.add_terms_to_whitelist(["not_malicious.com", "really_not_malicious.net"])
        self.assertEqual(len(indicators), 2)

        whitelist = self.ts.get_whitelist_page()
        self.assertGreater(len(whitelist), 0)

        for indicator in indicators:
            self.ts.delete_indicator_from_whitelist(indicator=indicator)

    def test_get_indicator_details(self):

        indicators = ["1.2.3.4", "evil.com"]

        report = Report(title="test",
                        body=' '.join(indicators))

        self.ts.submit_report(report)

        details = self.ts.get_indicator_details(indicators)

        print(details)

        self.ts.delete_report(report.id)

    def test_submit_indicators(self):
        indicators = [
            Indicator(value="1.5.8.7",
                      first_seen=get_current_time_millis() - MILLISECONDS_IN_A_DAY, last_seen=get_current_time_millis(),
                      sightings=100, source="Somewhere", notes="This is a note."),
            Indicator(value="1.5.8.9",
                      first_seen=get_current_time_millis() - MILLISECONDS_IN_A_DAY,
                      last_seen=get_current_time_millis(),
                      sightings=50, source="Somewhere else", notes="This is another note.")
        ]
        tags = [
            Tag(name="tag_1", enclave_id=self.ts.enclave_ids[0]),
            Tag(name="tag_2", enclave_id=self.ts.enclave_ids[0])
        ]
        self.ts.submit_indicators(indicators=indicators, tags=tags)

    def test_get_indicators(self):
        now = get_current_time_millis()
        from_time = now - 60
        indicators = self.ts.get_indicators(from_time=from_time, to_time=now)
        print(list(indicators))

    def test_get_indicator_metadata(self):
        metadata = self.ts.get_indicator_metadata("blah.com")
        print(metadata)

    def test_get_indicators_metadata(self):
        values = ['1.5.8.7', '1.5.8.9']

        indicators = [Indicator(value=value, type=IndicatorType.IP) for value in values]
        metadata = self.ts.get_indicators_metadata(indicators)
        self.assertTrue(len(metadata) == 2)

        for i in indicators:
            i.type = IndicatorType.URL

        metadata = self.ts.get_indicators_metadata(indicators)
        self.assertTrue(len(metadata) == 0)

        for i in indicators:
            i.type = None

        metadata = self.ts.get_indicators_metadata(indicators)
        self.assertTrue(len(metadata) == 2)

    def test_add_tag_to_indicator(self):
        tag = self.ts.add_indicator_tag("blah.com", name="indicator_tag", enclave_id=self.ts.enclave_ids[0])
        print(tag)

    def test_get_indicator_tags(self):
        tags = self.ts.get_all_indicator_tags()
        print(tags)

    def test_delete_tag_from_indicator(self):
        metadata = self.ts.get_indicator_metadata("blah.com")
        tags = metadata.get('tags')
        tag = tags[0]
        self.ts.delete_indicator_tag(metadata.get('indicator').value, tag_id=tag.id)

    def test_copy_report(self):

        # create and submit report
        original_report = Report(title="Report 1",
                                 body="Blah blah blah",
                                 time_began=yesterday_time,
                                 enclave_ids=self.ts.enclave_ids)
        original_tag = 'original_tag'
        original_report = self.ts.submit_report(report=original_report)
        self.ts.add_enclave_tag(original_report.id, original_tag, None)

        # find an enclave the user has write access to other than the enclave of the original report
        dest_enclave_id = None
        enclaves = self.ts.get_user_enclaves()
        for enclave in enclaves:
            if enclave.id != self.ts.enclave_ids[0] and enclave.update:
                dest_enclave_id = enclave.id
                break

        if not dest_enclave_id:
            raise Exception("User does not have access to any other enclaves to copy report to")

        # copy the report (without changes)
        copied_report_id = self.ts.copy_report(src_report_id=original_report.id,
                                               dest_enclave_id=dest_enclave_id,
                                               from_provided_submission=False)

        # find the copied report and check that the fields are equal to the original
        time.sleep(1)
        copied_report = self.ts.get_report_details(copied_report_id)
        original_tags = self.ts.get_enclave_tags(copied_report_id)

        self.assertEqual(original_report.title, copied_report.title)
        self.assertEqual(original_report.body, copied_report.body)
        self.assertEqual(copied_report.enclave_ids[0], dest_enclave_id)
        self.assertSetEqual({original_tag}, set(t.name for t in original_tags))

        # cleanup the copied report
        self.ts.delete_report(copied_report_id)

        # create an edited report to copy from
        changed_report = Report(title='Changed Title',
                                body='Changed body',
                                time_began=current_time,
                                enclave_ids=[dest_enclave_id])
        changed_tags = ['changed_tag']

        # copy report from a provided submission
        copied_report_id = self.ts.copy_report(src_report_id=original_report.id,
                                               dest_enclave_id=dest_enclave_id,
                                               from_provided_submission=True,
                                               report=changed_report,
                                               tags=changed_tags)

        # find the copied report and check that the fields are equal to the original
        time.sleep(1)
        copied_report = self.ts.get_report_details(copied_report_id)
        copied_tags = self.ts.get_enclave_tags(copied_report_id)

        self.assertEqual(copied_report.title, changed_report.title)
        self.assertEqual(copied_report.body, changed_report.body)
        self.assertEqual(copied_report.enclave_ids[0], dest_enclave_id)
        self.assertSetEqual(set(changed_tags), set(t.name for t in copied_tags))

    def test_move_report(self):

        # create and submit report
        original_report = Report(title="Report 1",
                                 body="Blah blah blah",
                                 time_began=yesterday_time,
                                 enclave_ids=self.ts.enclave_ids)
        original_tag = 'original_tag'
        original_report = self.ts.submit_report(report=original_report)
        self.ts.add_enclave_tag(original_report.id, original_tag, None)

        # find an enclave the user has write access to other than the enclave of the original report
        dest_enclave_id = None
        enclaves = self.ts.get_user_enclaves()
        for enclave in enclaves:
            if enclave.id != self.ts.enclave_ids[0] and enclave.update:
                dest_enclave_id = enclave.id
                break

        if not dest_enclave_id:
            raise Exception("User does not have access to any other enclaves to copy report to")

        # move the report
        moved_report_id = self.ts.move_report(report_id=original_report.id,
                                              dest_enclave_id=dest_enclave_id)

        # find the copied report and check that the fields are equal to the original
        time.sleep(1)
        moved_report = self.ts.get_report_details(moved_report_id)
        original_tags = self.ts.get_enclave_tags(moved_report_id)

        self.assertEqual(original_report.title, moved_report.title)
        self.assertEqual(original_report.body, moved_report.body)
        self.assertEqual(moved_report.enclave_ids[0], dest_enclave_id)
        self.assertSetEqual({original_tag}, set(t.name for t in original_tags))

        # cleanup the copied report
        self.ts.delete_report(moved_report_id)

    def test_get_phishing_submissions_page(self):
        with patch('trustar.api_client.ApiClient.post') as mocked_post:
            mocked_post.return_value.status_code = 200
            mocked_post.return_value.json.return_value = {"items": [{"submissionId": "1234"}],
                                                          'responseMetadata': {'nextCursor': ''}}

            page = self.ts.get_phishing_submissions_page(normalized_triage_score=normalized_triage_scores,
                                                         status=statuses)
            self.assertIsInstance(page, CursorPage)
            self.assertIsInstance(page.items, list)
            self.assertIsInstance(page.response_metadata, dict)
            self.assertEqual(str({"items": [{"submissionId": "1234"}], 'responseMetadata': {'nextCursor': ''}}),
                             str(page.to_dict()))

    def test_get_phishing_indicators_page(self):
        with patch('trustar.api_client.ApiClient.post') as mocked_post:
            mocked_post.return_value.status_code = 200
            mocked_post.return_value.json.return_value = {'items': [
                {'indicatorType': 'IP', 'value': '220.178.71.156', 'sourceKey': 'alienvault_otx',
                 'normalizedSourceScore': 3}],
                'responseMetadata': {'nextCursor': ''}}

            page = self.ts.get_phishing_indicators_page(
                normalized_triage_score=normalized_triage_scores,
                normalized_source_score=normalized_source_scores,
                status=statuses)
        self.assertIsInstance(page, CursorPage)
        self.assertIsInstance(page.items, list)
        self.assertIsInstance(page.response_metadata, dict)
        self.assertEqual(str({'items': [
            {'indicatorType': 'IP', 'value': '220.178.71.156', 'sourceKey': 'alienvault_otx',
             'normalizedSourceScore': 3}],
            'responseMetadata': {'nextCursor': ''}}),
            str(page.to_dict()))

        if __name__ == '__main__':
            unittest.main()
